/*    nethackappui.cpp
 *
 * Nethack UI Application UI class
 *
 * Copyright 2002-2004 Petteri Kangaslampi
 *
 * See license.txt for full copyright and license information.
*/

#include <eikenv.h>
#include <eikon.hrh>
#include <ckninfo.h>
#include <cknconf.h>
#include <eikmenup.h>
#include <eikdoc.h>
#include <eikapp.h>
#include <apgwgnam.h>
#include <apgtask.h>
#include <apgcli.h>
#include <apacmdln.h>
#include <e32hal.h>

#include "nethackappui.h"
#include "nethackappview.h"
#include "nethackui.hrh"
#include <nethack.rsg>

_LIT(KAssertPanic, "nethackappui.cpp");
#define assert(x) __ASSERT_ALWAYS(x, User::Panic(KAssertPanic, __LINE__))

#ifdef __WINS__
_LIT(KEngineFile, "nethackengine.dll");
#else
_LIT(KEngineFile, "nethackengine.exe");
#endif


//#include <e32svr.h>
//#define DEBUGPRINT(x) RDebug::Print x
#define DEBUGPRINT(x)


CNethackAppUi::CNethackAppUi() {
    iTermWidth = 80;
    iTermHeight = 24;
    iLargeFont = EFalse;
}


void CNethackAppUi::ConstructL() {

    User::CompressAllHeaps();
    BaseConstructL();

    iAppView = new (ELeave) CNethackAppView(this);
    iAppView->ConstructL(ClientRect());
    AddToStackL(iAppView);

    iAppView->Terminal()->SetGrayed(ETrue);

    iServer = CUiServer::NewL(*this, *iAppView->Terminal());    
}


CNethackAppUi::~CNethackAppUi() {
    delete iServer;
    delete iAppView;
}


void CNethackAppUi::HandleCommandL(TInt aCommand) {

    switch (aCommand) {

        case ENethackCmdStartEngine: {
            if ( !iServer ) {
                iServer = CUiServer::NewL(*this, *iAppView->Terminal());    
            }
            
            // Determine the path to the engine EXE
            TFileName appDll = iDocument->Application()->AppFullName();
            TParse parsa;
            User::LeaveIfError(parsa.SetNoWild(appDll, NULL, NULL));
            TFileName engineExe = parsa.DriveAndPath();
            engineExe.Append(KEngineFile);

            // Connect to launch server
            RApaLsSession ls;
            User::LeaveIfError(ls.Connect());
            CleanupClosePushL(ls);    

            // Build a command line to start the engine and start it
            CApaCommandLine *cl = CApaCommandLine::NewLC();
            cl->SetCommandL(EApaCommandRun);
            cl->SetLibraryNameL(engineExe);
            User::LeaveIfError(ls.StartApp(*cl));

            CleanupStack::PopAndDestroy(2);
            break;
        }

        case ENethackCmdMemoryStatus:
            ShowMemoryStatusL();
            break;

	case EEikCmdExit: 
            Exit();
            break;
    }    
}


void CNethackAppUi::ShowMemoryStatusL() {
    
    CEikonEnv *env = CEikonEnv::Static();
    HBufC *title = env->AllocReadResourceLC(R_STR_MEMORY_TITLE);
    HBufC *fmt = env->AllocReadResourceLC(R_STR_MEMORY_INFO);
    
    TBuf<128> buf;
    TMemoryInfoV1Buf memory;
    UserHal::MemoryInfo(memory);

    buf.Format(*fmt, memory().iMaxFreeRamInBytes, memory().iFreeRamInBytes);
    CCknInfoDialog::RunDlgLD(*title, buf);

    CleanupStack::PopAndDestroy(2); // fmt, title
}


void CNethackAppUi::MusoSessionStarted() {
    iEikonEnv->InfoMsg(_L("Session started"));
}


void CNethackAppUi::MusoSessionTerminated() {
}


void CNethackAppUi::MusoErrorMessage(const TDesC &aMessage) {
    HBufC *title = CEikonEnv::Static()->AllocReadResourceLC(R_STR_ERROR_TITLE);
    CCknInfoDialog::RunDlgLD(*title, aMessage);
    CleanupStack::PopAndDestroy();    
}


void CNethackAppUi::MusoFatalError(TInt aError) {
    User::Panic(_L("Fatal error"), aError);
}
